/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifndef _DATABASE_H_
#define _DATABASE_H_

/*
 * $Id: database.h,v 1.8 2003/03/05 05:05:54 rlaager Exp $
 */

#if HAVE_TIME_H
#include <time.h>
#endif

#define KD_ADD_VERBOSE 0x100000
#define KD_ADD_NO_STRIP_DISABLED 0x200000

int kd_add (unsigned char *keys, long len, int flags,
	    unsigned char **ret, long *retlen,
	    unsigned char **newkeys, long *newkeyslen);

#define KD_SEARCH_EXACT 0x1
#define KD_SEARCH_ALL 0x4
#define KD_SEARCH_IGNORE_ERRORS 0x8
#define KD_SEARCH_RETURN_DISABLED 0x10
#define KD_SEARCH_STDOUT 0x20

#define KD_SEARCH_FLAGS (KD_SEARCH_EXACT|KD_SEARCH_ALL|\
			 KD_SEARCH_IGNORE_ERRORS|KD_SEARCH_RETURN_DISABLED|\
			 KD_SEARCH_STDOUT)

#define KD_GET_EXACT KD_SEARCH_EXACT
#define KD_GET_ALL KD_SEARCH_ALL
#define KD_GET_IGNORE_ERRORS KD_SEARCH_IGNORE_ERRORS
#define KD_GET_RETURN_DISABLED KD_SEARCH_RETURN_DISABLED
#define KD_GET_STDOUT KD_SEARCH_STDOUT
#define KD_GET_MR 0x400
#define KD_GET_BINARY 0x1000

int kd_get (unsigned char *userid, long len, int flags, int mrk,
	    unsigned char **ret, long *retlen);

#define KD_INDEX_EXACT KD_SEARCH_EXACT
#define KD_INDEX_ALL KD_SEARCH_ALL
#define KD_INDEX_IGNORE_ERRORS KD_SEARCH_IGNORE_ERRORS
#define KD_INDEX_STDOUT KD_SEARCH_STDOUT
#define KD_INDEX_VERBOSE 0x100
#define KD_INDEX_FINGERPRINT 0x200
#define KD_INDEX_MR 0x400

int kd_index (unsigned char *userid, long len, int flags, int mrk,
	      unsigned char **ret, long *retlen);

#define KD_SINCE_BINARY 0x10000

int kd_since (time_t since, int flags, int mrk, time_t * last,
	      unsigned char **ret, long *retlen);

int kd_delete (unsigned char *userid, long len, int flags,
	       unsigned char **ret, long *retlen);

#define KD_DISABLE_CLEAR 0x100000

int kd_disable (unsigned char *userid, long len, int flags,
		unsigned char **ret, long *retlen);

int kd_create (char *dbdir, int num_files, int dbtype, char **ret);

#define KD_OPEN_NOTXN		0x1
#define KD_OPEN_READONLY	0x2
#define KD_OPEN_RECOVER		0x4

int kd_open (char *dbdir, int flags, char **ret);
int kd_sync ();
int kd_close ();

void kd_sig_handler ();
int kd_install_sig_handler (void (*h) ());

#endif
