/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

const char rcsid_kd_signal_c[] =
  "$Id: kd_signal.c,v 1.3 2003/02/28 17:17:16 rlaager Exp $";

/* this is necessary on some platforms (solaris) for sigaction()
   to work */
#define _POSIX_SOURCE

#if HAVE_SIGNAL_H
#include <signal.h>
#endif

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#include "database.h"
#include "globals.h"
#include "multiplex.h"

void
kd_sig_handler ()
{
  mp_terminate (EXIT_SIGNAL);
}

int
kd_install_sig_handler (void (*h) ())
{
  struct sigaction sa;

  sa.sa_flags = 0;
  sigemptyset (&(sa.sa_mask));
  sa.sa_handler = h;

  if (sigaction (SIGINT, &sa, NULL) < 0)
    return (0);

  if (sigaction (SIGTERM, &sa, NULL) < 0)
    return (0);

  if (sigaction (SIGHUP, &sa, NULL) < 0)
    return (0);

  if (sigaction (SIGQUIT, &sa, NULL) < 0)
    return (0);

  return (1);
}
