/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifndef _KD_TYPES_H_
#define _KD_TYPES_H_

#if HAVE_TIME_H
#include <time.h>
#endif

#include "llist.h"
#include "util.h"

/*
 * $Id: kd_types.h,v 1.6 2003/02/28 17:17:16 rlaager Exp $
 */

typedef struct _words_elem
{
  unsigned char *ptr;		/* ptr into userid xbuffer */
  long len;
}
words_elem;

typedef struct _sigs_elem
{
  xbuffer keyid;
  xbuffer sig;
  time_t sig_time;
}
sigs_elem;

typedef struct _userids_elem
{
  xbuffer uid;
  unsigned char *uidprint;
  long uidplen;
  llist sigs;
}
userids_elem;

typedef struct _keys_elem
{
  xbuffer pubkey;
  xbuffer subkey;
  xbuffer subkeysig;
  int modsigbits;
  xbuffer modbits, expbits;	/* used only for RSA fingerprint computation */
  xbuffer keyidbits;
  int keytype;
  int keyversion;
  time_t create_time;
  int disabled;
  userids_elem *primary;
  xbuffer revocation;
  llist userids;
  llist words;
}
keys_elem;

void sigs_elem_alloc (sigs_elem * se);
void userids_elem_alloc (userids_elem * ue);
void keys_elem_alloc (keys_elem * ke);
int malloc_elem_free (void *e, void *c);
int sigs_elem_free (void *e, void *c);
int userids_elem_free (void *e, void *c);
int keys_elem_free (void *e, void *c);
int bytestr_order (const unsigned char *s1, long s1len,
		   const unsigned char *s2, long s2len);
int words_elem_order (const void *e1, const void *e2);
int sigs_elem_order (const void *e1, const void *e2);
int userids_elem_order (const void *e1, const void *e2);
int keys_elem_order (const void *e1, const void *e2);

#endif
