/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

const char rcsid_kxa_c[] =
  "$Id: kxa.c,v 1.3 2003/02/28 17:17:17 rlaager Exp $";

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

#if HAVE_TIME_H
#include <time.h>
#endif

#include "armor.h"
#include "pgpfile.h"
#include "pgputil.h"
#include "util.h"

struct state
{
  xbuffer xb;
};

int
kxa (ddesc * packet, void *c)
{
  long ptype, plen;
  struct state *s = (struct state *) c;

  if (!decode_psf (packet, &ptype, &plen))
    return (0);

  if ((ptype == 6) || (ptype == 13) || (ptype == 2) || (ptype == 14))
    return (xbuffer_append (&(s->xb), packet->data, packet->size));
  else
    return (1);
}

int
do_file (ddesc * data)
{
  struct state s;
  ddesc binary, ascii;

  xbuffer_alloc (&(s.xb));

  if (!decode_file (data, kxa, (void *) &s))
    {
      xbuffer_free (&(s.xb));
      return (0);
    }

  binary.data = s.xb.buf;
  binary.size = s.xb.len;
  binary.offset = 0;

  ascii.size = encode_ascii_size (&binary, "PUBLIC KEY BLOCK");
  if ((ascii.data = (unsigned char *) malloc (ascii.size)) == NULL)
    {
      xbuffer_free (&(s.xb));
      return (0);
    }
  ascii.offset = 0;

  if (!encode_ascii (&binary, "PUBLIC KEY BLOCK", &ascii))
    {
      free (ascii.data);
      xbuffer_free (&(s.xb));
      return (0);
    }

  printf ("%.*s", (int) ascii.size, ascii.data);

  free (ascii.data);
  xbuffer_free (&(s.xb));

  return (1);
}
