/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifndef _LLIST_H_
#define _LLIST_H_

/*
 * $Id: llist.h,v 1.3 2003/02/28 17:17:19 rlaager Exp $
 */

#include "util.h"

#define LLIST_MERGE_DEL_1  0x1
#define LLIST_MERGE_DEL_2  0x2
#define LLIST_MERGE_FAIL   0x4

typedef struct _llist
{
  xbuffer xb;
}
llist;

typedef int (*llist_iter) (void *e, void *c);
/* This is the same as what qsort() and bsearch() take */
typedef int (*llist_order) (const void *e1, const void *e2);
typedef int (*llist_merger) (llist * lout, void *e1, void *e2, void *c);

void llist_alloc (llist * l);
long llist_count (llist * l);
int llist_add (llist * l, void *e);
/* if this returns -1, then the element was already present,
   and should be free'd */
int llist_add_sorted (llist * l, void *e, llist_order o);
int llist_copy (llist * dst, llist * src);
int llist_sort (llist * l, llist_order o);
int llist_iterate (llist * l, llist_iter i, void *c);
void llist_free (llist * l);

/* Merges l1 and l2 destructively into lout.

   While there are elements in l1 and l2, m is called on the first
   element of each list.  Once one list's elements are exhausted,
   NULL is passed instead of an element pointer from that list.  The
   merge terminates when l1 and l2 are empty.

   m should look at the two elements passed to it.  It may do nothing,
   add one or the other to lout, or combine the two elements and add
   that to lout.  The return value must request that one or both elements
   be deleted from their respective lists.  The elements should be freed
   inside m if they are not used.

   If an error occurs, all remaining elements will be passed to i, if
   i is non-null. The merge function will then return 0, with all
   lists empty, and need not be freed
*/

int llist_merge (llist * lout, llist * l1, llist * l2, llist_merger m,
		 void *mc, llist_iter i, void *ic);

#endif
