/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

const char rcsid_logging_c[] =
  "$Id: logging.c,v 1.6 2003/04/27 02:46:45 rlaager Exp $";

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

#if HAVE_STRING_H
#include <string.h>
#endif

#if HAVE_SYSLOG_H
#include <syslog.h>
#endif

#if HAVE_TIME_H
#include <time.h>
#endif

#include "globals.h"
#include "database.h"

#if !DEBUG
#define debug 0
#endif

int log_terminal;

static char *
current_time ()
{
  time_t t;
  char *time_str;
  t = time (NULL);
  time_str = ctime (&t);
  time_str[strlen (time_str) - 1] = '\0';
  return time_str;

}

void
log_debug (const char *fct, const char *msg)
{
  if (debug || log_terminal)
    fprintf (stdout, "[%s] %s: %s\n", current_time (), fct, msg);
  else
    syslog (LOG_DEBUG, "pksd: %s: %s", fct, msg);
}

void
log_info (const char *fct, const char *msg)
{
  if (debug || log_terminal)
    fprintf (stderr, "[%s] %s: %s\n", current_time (), fct, msg);
  else
    syslog (LOG_INFO, "pksd: %s: %s", fct, msg);
}

void
log_error (const char *fct, const char *msg)
{
  if (debug || log_terminal)
    fprintf (stderr, "[%s] %s: %s\n", current_time (), fct, msg);
  else
    syslog (LOG_ERR, "pksd: %s: %s", fct, msg);
}

void
log_fatal (const char *fct, const char *msg)
{
  /* try to keep the database intact */
  kd_close ();

  if (debug || log_terminal)
    fprintf (stderr, "[%s] FATAL ERROR in %s: %s\n", current_time (), fct,
	     msg);
  else
    syslog (LOG_CRIT, "pksd: %s: %s", fct, msg);

  exit (1);
}
