/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

const char rcsid_parse_c[] =
  "$Id: parse.c,v 1.3 2003/02/28 17:17:25 rlaager Exp $";

#if HAVE_CTYPE_H
#include <ctype.h>
#endif

#include "parse.h"
#include "util.h"

/* the return value is the number of characters in string up to
   and not including the char */

long
scan_char (unsigned char *input, long len, long *offset, unsigned char ch,
	   long *str, long *str_len)
{
  int i;

  if (*offset == len)
    {
      *str = -1;
      *str_len = 0;
      return (0);
    }

  for (i = *offset; i < len; i++)
    if (input[i] == ch)
      break;

  if (i == len)
    {
      /* end of input reached before ch.  don't return a string
         or move the pointer, but return the number of chars seen */
      *str = -1;
      *str_len = 0;
      return (i - *offset);
    }

  i++;

  *str = *offset;
  *str_len = i - *offset;

  *offset = i;

  return (*str_len - 1);
}

void
scan_token (unsigned char *input, long len, long *offset,
	    long *token, long *token_len)
{
  int i;

  if (*offset == len)
    {
      *token = -1;
      *token_len = 0;
      return;
    }

  /* scan for end of token */

  for (i = *offset; i < len && !isspace (input[i]); i++)
    /* SUPPRESS 570 */
    ;

  *token = *offset;
  *token_len = i - *offset;

  *offset = i;

  return;
}

void
scan_space (unsigned char *input, long len, long *offset)
{
  int i;

  for (i = *offset; i < len && (input[i] != '\n') && isspace (input[i]); i++)
    /* SUPPRESS 570 */
    ;

  *offset = i;

  return;
}

/* the return value is the number of characters in the line
   not including trailing CR*LF */

long
scan_line (unsigned char *input, long len, long *offset,
	   long *line, long *line_len)
{
  int cnt;
  long tmp;

  cnt = scan_char (input, len, offset, '\012', line, line_len);

  /* strip off trailing CR */

  tmp = (*line >= 0) ? *line : *offset;

  while ((cnt > 0) && (input[tmp + cnt - 1] == '\015'))
    cnt--;

  return (cnt);
}

int
is_token (const unsigned char *str, long strlen,
	  const char *token, long toklen, int exact)
{
  if (strlen < toklen)
    return (0);

  if (exact && (strlen != toklen))
    return (0);

  return (my_strncasecmp ((char *) str, (char *) token, (int) toklen) == 0);
}
