/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

const char rcsid_pgpcrc_c[] =
  "$Id: pgpcrc.c,v 1.3 2003/02/28 17:17:25 rlaager Exp $";

#include "pgpcrc.h"
#include "pgputil.h"

#define CRC_GENERATOR 0x864CFB
#define CRC_INIT 0xB704CE
#define CRC_BITS 24

int crc_byte_initted;
unsigned long crc_byte[256];

static void
init_crc_byte ()
{
  int i;

  crc_byte[0] = 0;

  for (i = 1; i < 256; i++)
    crc_byte[i] = (((crc_byte[i >> 1]) << 1)
		   ^
		   (((crc_byte[i >> 1] >> (CRC_BITS - 1)) ^ (i & 0x1)) ?
		    CRC_GENERATOR : 0)) & ((1 << CRC_BITS) - 1);
}

void
crc_compute (ddesc * data, long *crc)
{
  int i;
  long tmp;

  if (!crc_byte_initted)
    {
      init_crc_byte ();
      crc_byte_initted = 1;
    }

  tmp = CRC_INIT;

  for (i = 0; i < data->size; i++)
    tmp =
      (tmp << 8) ^ crc_byte[(data->data[i] ^ (tmp >> (CRC_BITS - 8))) & 0xff];

  *crc = tmp & ((1 << CRC_BITS) - 1);
}
