/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

const char rcsid_pgpsplit_c[] =
  "$Id: pgpsplit.c,v 1.8 2003/03/03 21:12:07 rlaager Exp $";

#if HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

#if HAVE_STRING_H
#include <string.h>
#endif

#if HAVE_TIME_H
#include <time.h>
#endif

#include "pgpfile.h"
#include "pgputil.h"
#include "util.h"

struct state
{
  int maxbytes;
  char *filenamebase;

  int filenum;
  char filename[MAXPATHLEN];
  FILE *f;

  int bytes;
};

int
split (ddesc * packet, void *c)
{
  long ptype, plen;
  struct state *s = (struct state *) c;

  if (!decode_psf (packet, &ptype, &plen))
    return (0);

  if ((ptype == 6) && (s->f) && ((s->bytes + packet->size) >= s->maxbytes))
    {
      fclose (s->f);
      s->f = NULL;
    }

  if (!s->f)
    {
      sprintf (s->filename, "%s%04d.pgp", s->filenamebase, s->filenum++);
      if ((s->f = fopen (s->filename, "w")) == NULL)
	{
	  fprintf (stderr, "Error opening %s for writing\n", s->filename);
	  return (0);
	}
      s->bytes = 0;
    }

  if (fwrite (packet->data, packet->size, 1, s->f) != 1)
    {
      fprintf (stderr, "Error writing to %s\n", s->filename);
      return (0);
    }

  s->bytes += packet->size;

  return (1);
}

void
usage (char *argv0)
{
  fprintf (stderr, "Usage: %s [-b bytes] [-o outfile] infile\n", argv0);
  exit (1);
}

int
main (int argc, char *argv[])
{
  int i;
  FILE *f;
  char *infile, *outfile;
  struct state s;
  xfilecontents xfc;
  ddesc data;

  infile = NULL;
  outfile = NULL;

  s.maxbytes = 0;
  s.filenamebase = NULL;
  s.f = NULL;
  s.filenum = 0;

  for (i = 1; i < argc; i++)
    {
      if (argv[i][0] == '-')
	{
	  if (argv[i][1] == 'b')
	    {
	      if (s.maxbytes || !argv[i + 1])
		usage (argv[0]);

	      if ((s.maxbytes = atoi (argv[i + 1])) <= 0)
		usage (argv[0]);

	      i++;
	    }
	  else if (argv[i][1] == 'o')
	    {
	      if (outfile || !argv[i + 1])
		usage (argv[0]);

	      outfile = argv[i + 1];

	      i++;
	    }
	  else
	    {
	      usage (argv[0]);
	    }
	}
      else
	{
	  if (infile)
	    usage (argv[0]);

	  infile = argv[i];
	}
    }

  if (!infile && !outfile)
    usage (argv[0]);

  if (!s.maxbytes)
    s.maxbytes = 10 * 1024 * 1024;

  if (outfile)
    s.filenamebase = outfile;
  else
    s.filenamebase = infile;

  if (!infile)
    {
      infile = "<STDIN>";
      f = stdin;
    }
  else
    {
      if ((f = fopen (infile, "r")) == NULL)
	{
	  perror ("opening file");
	  exit (1);
	}
    }

  if (!xfilecontents_get (&xfc, f))
    {
      perror ("reading file");
      exit (1);
    }

  data.data = xfc.buf;
  data.size = xfc.len;
  data.offset = 0;

  if (!decode_file (&data, split, (void *) &s))
    {
      perror ("reading file");
      xfilecontents_free (&xfc);
      exit (1);
    }

  xfilecontents_free (&xfc);
  return (0);
}
