/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifndef _PGPUTIL_H_
#define _PGPUTIL_H_

/*
 * $Id: pgputil.h,v 1.5 2003/02/28 17:17:27 rlaager Exp $
 */

#if HAVE_TIME_H
#include <time.h>
#endif

typedef struct _ddesc
{
  unsigned char *data;
  long size;
  long offset;
}
ddesc;

typedef struct _mpidesc
{
  long nbits;
  ddesc number;
}
mpidesc;

#define DECODE_READABLE(data, length) \
	((data)->size >= ((data)->offset + (length)))

typedef int (*packet_handler) (ddesc * packet, void *closure);

int decode_num (ddesc * data, long size, long *num);
int decode_psf (ddesc * data, long *ptype, long *plen);
int decode_pubkey (ddesc * data, long len, mpidesc * modulus,
		   mpidesc * exponent, unsigned char *keyid,
		   unsigned char *keytype, time_t * create_time,
		   int *keyversion);
int decode_userid (ddesc * data, long len, ddesc * userid);
int decode_sig (ddesc * data, long len, ddesc * keyid, long *sigclass,
		time_t * sigtime);
int decode_file (ddesc * data, packet_handler h, void *c);
int decode_binary (ddesc * data, packet_handler h, void *c);

#endif
