/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

const char rcsid_pks_c[] =
  "$Id: pks_config.c,v 1.15 2003/04/27 02:46:45 rlaager Exp $";

#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#if HAVE_CTYPE_H
#include <ctype.h>
#endif

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

#if HAVE_STRING_H
#include <string.h>
#endif

#include "globals.h"
#include "parse.h"
#include "pks_config.h"
#include "util.h"

const pks_config default_config = {
  "",
  "",
  0,
  0,
  "/var/spool/pks/etc/db",
  "/var/lib/pks",
  0,
  {{0, 0, NULL}},
  1,
  "/var/spool/pks/pksd_socket",
  "",
  "root",
  "/var/spool/pks/etc/mail_intro",
  "/var/spool/pks/etc/mail_help",
  "EN",
  "",
  /* okay, it's an abstraction violation.  I wouldn't need to do
     this in C++ */
  {{0, 0, NULL}},
  -1,
  -1,
};

const char pid_dir_str[] = "pid_dir";
long pid_dir_len = sizeof (pid_dir_str) - 1;

const char chroot_dir_str[] = "chroot_dir";
long chroot_dir_len = sizeof (chroot_dir_str) - 1;

const char pksd_uid_str[] = "pksd_uid";
long pksd_uid_len = sizeof (pksd_uid_str) - 1;

const char pksd_gid_str[] = "pksd_gid";
long pksd_gid_len = sizeof (pksd_gid_str) - 1;

const char db_dir_str[] = "db_dir";
long db_dir_len = sizeof (db_dir_str) - 1;

const char www_dir_str[] = "www_dir";
long www_dir_len = sizeof (www_dir_str) - 1;

const char www_port_str[] = "www_port";
long www_port_len = sizeof (www_port_str) - 1;

const char www_addr_str[] = "www_addr";
long www_addr_len = sizeof (www_addr_str) - 1;

const char www_readonly_str[] = "www_readonly";
long www_readonly_len = sizeof (www_readonly_str) - 1;

const char sockname_str[] = "socket_name";
long sockname_len = sizeof (sockname_str) - 1;

const char mdc_str[] = "mail_delivery_client";
long mdc_len = sizeof (mdc_str) - 1;

const char maint_str[] = "maintainer_email";
long maint_len = sizeof (maint_str) - 1;

const char mif_str[] = "mail_intro_file";
long mif_len = sizeof (mif_str) - 1;

const char helpdir_str[] = "help_dir";
long helpdir_len = sizeof (helpdir_str) - 1;

const char deflang_str[] = "default_language";
long deflang_len = sizeof (deflang_str) - 1;

const char this_site_str[] = "this_site";
long this_site_len = sizeof (this_site_str) - 1;

const char syncsite_str[] = "syncsite";
long syncsite_len = sizeof (syncsite_str) - 1;

const char max_last_str[] = "max_last";
long max_last_len = sizeof (max_last_str) - 1;

const char max_last_reply_keys_str[] = "max_last_reply_keys";
long max_last_reply_keys_len = sizeof (max_last_reply_keys_str) - 1;

const char max_reply_keys_str[] = "max_reply_keys";
long max_reply_keys_len = sizeof (max_reply_keys_str) - 1;

#if DEBUG
const char debug_str[] = "debug";
long debug_len = sizeof (debug_str) - 1;
#endif

static int
compare_strings (const void *e1, const void *e2)
{
  return (my_strncasecmp ((const char *) e1, (const char *) e2, -1));
}

static int
free_string (void *s, void *c)
{
  free (s);

  return (1);
}

/* this is cpp black magic */

#define noop() /**/
#define num_conf(vptr, vlen, confvar) \
      /* if ( */ is_token(line+word, word_len, (vptr), (vlen), 1)) { \
	 scan_token(line, len, &offset, &word, &word_len); \
	 (confvar) = atoi((char *) line+word); \
      } noop(
#define str_conf(vptr, vlen, confvar) \
      /* if ( */ is_token(line+word, word_len, (vptr), (vlen), 0)) { \
	 scan_space(line, len, &offset); \
	 word = offset; \
	 word_len = len-offset; \
	 while (word_len && isspace(line[word+word_len-1])) \
	    word_len--; \
	 strncpy((confvar), (char *) line+word, (size_t) word_len); \
	 (confvar)[word_len] = '\0'; \
      } noop(
#define llist_conf(vptr, vlen, confvar) \
      /* if ( */ is_token(line+word, word_len, (vptr), (vlen), 0)) { \
	 scan_space(line, len, &offset); \
	 word = offset; \
	 word_len = len-offset; \
	 while (word_len && isspace(line[word+word_len-1])) \
	 word_len--; \
         { \
	    char *str; \
	    if (((str = (char *) malloc(word_len+1)) == NULL) || \
		(strncpy(str, (char *) line+word, (size_t) word_len), \
		 (str[word_len] = '\0'), \
		 !llist_add_sorted((confvar), str, compare_strings))) { \
	       llist_iterate((confvar), free_string, NULL); \
	       llist_free((confvar)); \
	       return(0); \
	    } \
	 } \
      } noop(
  int
pks_read_conf_file (const char *filename, pks_config * pc)
{
  FILE *conf;
  char buf[1024];
  unsigned char *line;
  long len, offset, word, word_len;

  *pc = default_config;

  if ((conf = fopen (filename, "r")) == NULL)
    return (0);

  while ((line = (unsigned char *) fgets (buf, sizeof (buf), conf)))
    {
      if (line[0] == '#')
	continue;

      len = strlen ((char *) line);
      offset = 0;

      scan_token (line, len, &offset, &word, &word_len);
      scan_space (line, len, &offset);

      if (str_conf (pid_dir_str, pid_dir_len, pc->pid_dir))
	else
      if (str_conf (chroot_dir_str, chroot_dir_len, pc->chroot_dir))
	else
      if (num_conf (pksd_uid_str, pksd_uid_len, pc->pksd_uid))
	else
      if (num_conf (pksd_gid_str, pksd_gid_len, pc->pksd_gid))
	else
      if (str_conf (db_dir_str, db_dir_len, pc->db_dir))
	else
      if (str_conf (www_dir_str, www_dir_len, pc->www_dir))
	else
      if (llist_conf (www_addr_str, www_addr_len, &(pc->www_addrs)))
	else
      if (num_conf (www_port_str, www_port_len, pc->www_port))
	else
      if (num_conf (www_readonly_str, www_readonly_len, pc->www_readonly))
	else
      if (str_conf (sockname_str, sockname_len, pc->socket_name))
	else
      if (str_conf (mdc_str, mdc_len, pc->mail_delivery_client))
	else
      if (str_conf (maint_str, maint_len, pc->maintainer_email))
	else
      if (str_conf (mif_str, mif_len, pc->mail_intro_file))
	else
      if (str_conf (helpdir_str, helpdir_len, pc->help_dir))
	else
      if (str_conf (deflang_str, deflang_len, pc->default_lang))
	else
      if (str_conf (this_site_str, this_site_len, pc->this_site))
	else
      if (num_conf (max_last_str, max_last_len, pc->max_last))
	else
      if (num_conf (max_last_reply_keys_str, max_last_reply_keys_len,
		      pc->max_last_reply_keys))
	else
      if (num_conf (max_reply_keys_str, max_reply_keys_len,
		      pc->max_reply_keys))
	else
      if (llist_conf (syncsite_str, syncsite_len, &(pc->syncsites)))
#if DEBUG
	else
      if (num_conf (debug_str, debug_len, debug))
#endif
	}

	fclose (conf);

      return (1);
    }
