/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

const char rcsid_dbcheck_c[] =
  "$Id: pkscheck.c,v 1.8 2003/07/31 18:45:18 rlaager Exp $";

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

#if HAVE_STRING_H
#include <string.h>
#endif

#include "database.h"
#include "globals.h"
#include "kd_internal.h"

void
usage (char *argv0)
{
  fprintf (stderr, "%s /db/path\n", argv0);
  exit (1);
}

int
main (int argc, char *argv[])
{
  char *dir, *str;
  DBC *cursor;
  DBT ikey, idata, kkey, kdata;
  int ret, i;
  char buf[1024];

  if (argc < 2)
    usage (argv[0]);

  dir = argv[1];
  log_terminal = 1;

  if (!kd_open (dir, KD_OPEN_READONLY, &str))
    {
      fprintf (stderr, "database open failed: %s\n", str);
      free (str);
      exit (1);
    }

  if ((ret = (*(worddb->cursor)) (worddb, NULL, &cursor, 0)))
    {
      sprintf (buf, "error creating worddb cursor: error = %d", ret);
      log_error ("main", buf);
    }

  memset (&ikey, 0, sizeof (ikey));
  memset (&idata, 0, sizeof (idata));

  memset (&kkey, 0, sizeof (kkey));
  memset (&kdata, 0, sizeof (kdata));

  for (ret = (*(cursor->c_get)) (cursor, &ikey, &idata, DB_FIRST);
       ret == 0; ret = (*(cursor->c_get)) (cursor, &ikey, &idata, DB_NEXT))
    {
      if (idata.size != 12)
	{
	  sprintf (buf, "worddb corrupt in entry = \"%.*s\", size = %d\n",
		   (int) ikey.size, (char *) ikey.data, (int) idata.size);

	  log_error ("main", buf);
	}

      kkey.size = 4;
      kkey.data = ((unsigned char *) idata.data) + 8;

      if ((*(keydb (&kkey)->get)) (keydb (&kkey), NULL, &kkey, &kdata, 0))
	{
	  sprintf (buf, "keyid %02X%02X%02X%02X in worddb but not keydb\n",
		   ((unsigned char *) kkey.data)[0],
		   ((unsigned char *) kkey.data)[1],
		   ((unsigned char *) kkey.data)[2],
		   ((unsigned char *) kkey.data)[3]);
	  log_error ("main", buf);
	}
    }

  (*(cursor->c_close)) (cursor);

  if ((ret = (*(timedb->cursor)) (timedb, NULL, &cursor, 0)))
    {
      sprintf (buf, "error creating timedb cursor: error = %d", ret);
      log_error ("main", buf);
    }

  memset (&ikey, 0, sizeof (ikey));
  memset (&idata, 0, sizeof (idata));

  memset (&kkey, 0, sizeof (kkey));
  memset (&kdata, 0, sizeof (kdata));


  for (ret = (*(cursor->c_get)) (cursor, &ikey, &idata, DB_FIRST);
       ret == 0; ret = (*(cursor->c_get)) (cursor, &ikey, &idata, DB_NEXT))
    {
      if (idata.size % 12)
	{
	  sprintf (buf, "timedb corrupt in entry = \"%.*s\", size = %d\n",
		   (int) ikey.size, (char *) ikey.data, (int) idata.size);

	  log_error ("main", buf);
	}

      for (KD_FIRST_ENTRY (i); KD_LAST_ENTRY (i, idata); KD_NEXT_ENTRY (i))
	{
	  kkey.size = 4;
	  kkey.data = ((unsigned char *) idata.data) + i + 8;

	  if ((((unsigned long *) kkey.data)[0] > 0) && (*(keydb (&kkey)->get))
	      (keydb (&kkey), NULL, &kkey, &kdata, 0))
	    {
	      sprintf (buf,
		       "keyid %02X%02X%02X%02X in timedb but not keydb\n",
		       ((unsigned char *) kkey.data)[0],
		       ((unsigned char *) kkey.data)[1],
		       ((unsigned char *) kkey.data)[2],
		       ((unsigned char *) kkey.data)[3]);
	      log_error ("main", buf);
	    }
	}
    }

  (*(cursor->c_close)) (cursor);

  kd_close ();

  exit (0);
}
