/*
 * Portions copyright 2003 Richard Laager.
 * Copyright 2003 Fancisco Jesus Monserrat Coll.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

const char rcsid_pkskeydump_c[] =
  "$Id: pkskeydump.c,v 1.3 2003/02/28 17:17:39 rlaager Exp $";

#if HAVE_ERRNO_H
#include <errno.h>
#endif

#if HAVE_FCNTL_H
#include <fcntl.h>
#endif

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

#if HAVE_STRING_H
#include <string.h>
#endif

#include "database.h"
#include "globals.h"
#include "kd_internal.h"

void
usage (char *argv0)
{
  fprintf (stderr, "%s /db/path basename\n", argv0);
  exit (1);
}

void
dump_keyrings (char *basename)
{
  DBC *cursor;
  DBT key, data;
  int i, ret, count, bufsize;
  char buf[1024];
  FILE *f;
  char filename[1024];

  for (i = 0; i < num_keydb; i++)
    {
      if ((ret = (*(keydb_files[i]->cursor)) (keydb_files[i], NULL, &cursor,
					      0)))
	{

	  sprintf (buf, "error creating keydb[%d] cursor: error = %d", i,
		   ret);
	  log_error ("main", buf);
	  return;

	}
      sprintf (filename, "%s%04d", basename, i);

      memset (&key, 0, sizeof (key));
      memset (&data, 0, sizeof (data));

      bufsize = 0;
      buf[0] = '\0';
      count = 0;

      printf ("Dumping keyfile %d on %s\n", i, filename);
      if ((f = fopen (filename, "w")) == NULL)
	{
	  printf ("Error, I can't write PGPring %s\n", filename);
	  exit (-1);
	}
      /* Dumping of a keyring */
      for (ret = (*(cursor->c_get)) (cursor, &key, &data, DB_FIRST);
	   ret == 0; ret = (*(cursor->c_get)) (cursor, &key, &data, DB_NEXT))
	{


	  fwrite ((void *) data.data, (size_t) data.size, 1, f);
	}
      fclose (f);

      /* This is the end of the loop to dump 1 keyring file */


      (*(cursor->c_close)) (cursor);
    }

  /* This is the end of the loop  to dump the whole rings */

}


int
main (int argc, char *argv[])
{
  char *dir, *str;

  char basename[1024];
  if (argc < 3)
    usage (argv[0]);

  strncpy (basename, argv[2], 1024);

  dir = argv[1];
  log_terminal = 1;

  if (!kd_open (dir, KD_OPEN_READONLY, &str))
    {
      fprintf (stderr, "database open failed: %s\n", str);
      free (str);
      exit (1);
    }

  dump_keyrings (basename);


  kd_close ();

  exit (0);
}
