/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

const char rcsid_mailreq_c[] =
  "$Id: pksmailreq.c,v 1.5 2003/02/28 17:17:40 rlaager Exp $";

#if HAVE_SIGNAL_H
#include <signal.h>
#endif

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

#if HAVE_UNISTD_H
#include <unistd.h>
#endif

#include "database.h"
#include "globals.h"
#include "llist.h"
#include "mail_req.h"
#include "multiplex.h"
#include "pks_incr.h"
#include "util.h"

void
usage (const char *argv0)
{
  fprintf (stderr, "usage: %s\n", argv0);
  exit (1);
}

int
main (int argc, char *argv[])
{
  mail_send_conf msc;
  llist empty;
  pks_incr_conf pic;
  mail_req_conf mrc;
  xfilecontents xfc;
  char *ret;

  if (argc != 1)
    usage (argv[0]);

  /* this is kinda gross, but send_mail wants to steal cat's stdout, so
     I do this. */

  if (dup2 (1, 3) < 0)
    log_fatal ("main", "couldn't dup fd 1 onto fd 3");

  msc.mail_delivery_client = "cat 1>&3";
  msc.maintainer_email = "marc@mit.edu";
  msc.mail_intro_file = "/tmp/mail_intro_file";
  msc.help_dir = "/tmp/help_dir";
  msc.default_lang = "EN";

  pic.this_site = "";		/* won't ever be used */
  llist_alloc (&empty);
  pic.syncsites = &empty;
  pic.msc = &msc;

  mrc.msc = &msc;
  mrc.pic = &pic;

  mrc.max_last = 3;

  log_terminal = 1;

  if (!kd_install_sig_handler (&kd_sig_handler))
    log_fatal ("main", "installing kd_sig_handler");

  if (!mp_ignore_signals ())
    log_fatal ("main", "ignoring multiplexor signals");

  if (!kd_open ("/var/tmp/db", 0, &ret))
    log_fatal ("main", "database open failed");

  if (!mp_init ())
    log_fatal ("main", "failed initializing multiplexor");

  if (!xfilecontents_get (&xfc, stdin))
    log_fatal ("main", "error reading mail message from stdin");

  mail_req (xfc.buf, xfc.len, &mrc, NULL, NULL);

  xfilecontents_free (&xfc);

  if (!mp_go ())
    log_fatal ("main", "error in multiplexor main loop");

  kd_close ();

  exit (0);
}
