/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifndef _UTIL_H_
#define _UTIL_H_

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

/*
 * $Id: util.h,v 1.3 2003/02/28 17:17:42 rlaager Exp $
 */

typedef struct _xbuffer
{
  long alloclen, len;
  unsigned char *buf;
}
xbuffer;

typedef struct _xfilecontents
{
/* these are "public" */
  long len;
  unsigned char *buf;
/* these are "private" */
  xbuffer xb;
#ifdef HAVE_MMAP
  int mmapped;
#endif
}
xfilecontents;

extern const unsigned char zeros[];

void xbuffer_alloc (xbuffer * xb);
int xbuffer_append (xbuffer * xb, const unsigned char *data, long len);
int xbuffer_append_str (xbuffer * xb, const char *data);
void xbuffer_free (xbuffer * xb);

/* XXX easiest, certainly wrt error handling */
int xfilecontents_get (xfilecontents * xfc, FILE * f);
void xfilecontents_free (xfilecontents * xfc);

char *my_strdup (const char *s);
int my_strncasecmp (const char *s1, const char *s2, int n);
void *my_memmem (const void *mem, const void *s, size_t memn, size_t mems);
void *my_memcasemem (const void *mem, const void *s, size_t memn,
		     size_t mems);
void *my_memstr (const void *mem, const char *s, size_t n);

#endif
