/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

const char rcsid_www_c[] =
  "$Id: www.c,v 1.22 2003/07/31 18:45:18 rlaager Exp $";

#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#if HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif

#if HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif

#if HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif

#if HAVE_CTYPE_H
#include <ctype.h>
#endif

#if HAVE_FCNTL_H
#include <fcntl.h>
#endif

#if HAVE_ERRNO_H
#include <errno.h>
#endif

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

#if HAVE_STRING_H
#include <string.h>
#endif

#if HAVE_UNISTD_H
#include <unistd.h>
#endif

#include "globals.h"
#include "llist.h"
#include "multiplex.h"
#include "parse.h"
#include "pks_www.h"
#include "util.h"
#include "www.h"

#if HAVE_LIBWRAP

#if HAVE_NETDB_H
#include <netdb.h>
#endif

#if HAVE_SYSLOG_H
#include <syslog.h>
#endif

#include <tcpd.h>

#define TCPDSERVICE "pksd"

int allow_severity = LOG_WARNING;
int deny_severity = LOG_WARNING;
extern int hosts_ctl (char *daemon,
		      char *client_name, char *client_addr,
		      char *client_user);

#endif /* HAVE_LIBWRAP */

static const long hexchar[] = {
  256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
  256,
  256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
  256,
  256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
  256,
  0, -1, -2, -3, -4, -5, -6, -7, -8, -9, 256, 256, 256, 256, 256, 256,

  256, -10, -11, -12, -13, -14, -15, 256, 256, 256, 256, 256, 256, 256, 256,
  256,
  256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
  256,
  256, -10, -11, -12, -13, -14, -15, 256, 256, 256, 256, 256, 256, 256, 256,
  256,
  256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
  256,

  256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
  256,
  256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
  256,
  256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
  256,
  256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
  256,

  256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
  256,
  256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
  256,
  256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
  256,
  256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
  256,
};

struct www_listen_info
{
  int port;
  httphandler h;
  void *c;
};

int
www_urldecode (unsigned char *str, long *len)
{
  long ptr, i, tmp;

  for (i = 0, ptr = 0; i < *len; i++, ptr++)
    {
      if (str[i] == '%')
	{
	  if (i + 2 >= *len)
	    return (0);
	  if ((tmp = hexchar[str[i + 1]] * 16 + hexchar[str[i + 2]]) > 0)
	    return (0);
	  str[ptr] = -tmp;
	  i += 2;
	}
      else if (str[i] == '+')
	{
	  str[ptr] = ' ';
	}
      else
	{
	  str[ptr] = str[i];
	}
    }

  *len = ptr;

  return (1);
}

static const char content_length[] = "content-length:";
static int content_length_len = sizeof (content_length) - 1;

static const char get[] = "get";
static int get_len = sizeof (get) - 1;

static const char post[] = "post";
static int post_len = sizeof (post) - 1;

static void
writer (int fd, unsigned char *output, long len, void *c)
{
  xbuffer_free ((xbuffer *) c);
  free (c);
}

void
www_reply (int fd, int vers, int status_code, const char *reason_phrase,
	   const char *content_type,
	   const unsigned char *reply, long replylen)
{
  xbuffer *xb;
  char num[20];

  if (content_type == NULL)
    content_type = "text/html;charset=utf-8";

  if ((xb = (xbuffer *) malloc (sizeof (xbuffer))) == NULL)
    log_fatal ("www_reply", "failed allocating memory for xbuffer");

  xbuffer_alloc (xb);

  if (vers < 1000)
    {
      if (!xbuffer_append (xb, reply, replylen))
	{
	  xbuffer_free (xb);
	  log_fatal ("www_reply", "failed constructing www 0.9 reply");
	}
    }
  else
    {
      sprintf (num, "%d ", status_code);

      if (!xbuffer_append_str (xb, "HTTP/1.0 ") ||
	  !xbuffer_append_str (xb, num) ||
	  !xbuffer_append_str (xb, reason_phrase) ||
	  !xbuffer_append_str (xb, "\015\012") ||
	  !xbuffer_append_str (xb,
			       "Server: pks_www/" PACKAGE_VERSION "\015\012")
	  || !xbuffer_append_str (xb, "Content-type: ")
	  || !xbuffer_append_str (xb, content_type)
	  || !xbuffer_append_str (xb, "\015\012")
	  || !xbuffer_append_str (xb, "\015\012"))
	{
	  xbuffer_free (xb);
	  log_fatal ("www_reply", "failed constructing www reply header");
	}

      if ((status_code / 100) == 4)
	{
	  if (!xbuffer_append_str (xb,
				   "<?xml version=\"1.0\"?>\015\012"
				   "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\015\012"
				   "    \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\015\012"
				   "<html xmlns=\"http://www.w3.org/1999/xhtml\">\015\012"
				   "  <head>\015\012"
				   "    <title>") ||
	      !xbuffer_append_str (xb, num) ||
	      !xbuffer_append_str (xb, reason_phrase) ||
	      !xbuffer_append_str (xb, "</title>\015\012"
				   "  </head>\015\012"
				   "  <body>")
	      || !xbuffer_append (xb, reply, replylen)
	      || !xbuffer_append_str (xb,
				      "  </body>\015\012" "</html>\015\012"))
	    {
	      xbuffer_free (xb);
	      log_fatal ("www_reply", "failed constructing www error reply");
	    }
	}
      else
	{
	  if (!xbuffer_append (xb, reply, replylen))
	    {
	      xbuffer_free (xb);
	      log_fatal ("www_reply",
			 "failed constructing www success reply");
	    }
	}
    }

  if (!mp_add_write (fd, 1, xb->buf, xb->len, &writer, (void *) xb))
    log_fatal ("www_reply", "failed adding new output stream");
}

static const char bad_request[] = "Bad Request";

typedef struct _hh_state
{
  httphandler h;
  xbuffer xb;
  unsigned int readonly;
  void *c;
  long ptr;
  long method, uri, method_len, uri_len;
  long content_length, body;
}
hh_state;

static void
reader (int fd, unsigned char *input, long len, int done, void *c)
{

  hh_state *s = (hh_state *) c;
  long ptr, cnt;
  long dummy, dummy_len;
  unsigned int readonly = s->readonly;


  if (done < 0)
    {
      xbuffer_free (&(s->xb));
      free (s);
      return;
    }

  if (!xbuffer_append (&(s->xb), input, len))
    log_fatal ("reader [www]", "failed appending to www request buffer");

  input = s->xb.buf;
  len = s->xb.len;

  if (!len)
    {
      if (done == 0)
	return;

      /* if there's no input at end of file, that's an error */

      www_reply (fd, 1000, 400, bad_request, NULL, NULL, 0);
      mp_delete_read (fd);
      xbuffer_free (&(s->xb));
      free (s);
      return;
    }

   /*** parse an HTML request */

  if (s->uri < 0)
    {
      scan_token (input, len, &(s->ptr), &(s->method), &(s->method_len));
      scan_space (input, len, &(s->ptr));

      scan_token (input, len, &(s->ptr), &(s->uri), &(s->uri_len));
      scan_space (input, len, &(s->ptr));

      cnt = scan_line (input, len, &(s->ptr), &dummy, &dummy_len);

      /* if the first line isn't complete, and the stream has
         not ended, reset for next time */
      if ((dummy_len == 0) && (done == 0))
	{
	  s->ptr = 0;
	  s->method = -1;
	  s->uri = -1;

	  return;
	}

      /* the first line is done (either because it is, or because
         the input stream ended).
         if there's no method or no uri, that's an error */
      if ((s->method == -1) || (s->uri == -1))
	{
	  www_reply (fd, 1000, 400, bad_request, NULL, NULL, 0);
	  mp_delete_read (fd);
	  xbuffer_free (&(s->xb));
	  free (s);
	  return;
	}

      /* if there's nothing else on the first line, it's an 0.9 request */
      if (cnt == 0)
	{
	  s->body = 0;
	  s->content_length = 0;
	  /* fall all the way through to the request parser/handler */
	}

      /* the first line is now dealt with.  the next block deals with the
         headers, if any */
    }

  if (s->body < 0)
    {
      /* Scan headers, store away content-length, stop after blank line */

      while (1)
	{
	  cnt = scan_line (input, len, &(s->ptr), &dummy, &dummy_len);

	  /* check for complete line */

	  if (dummy_len)
	    {
	      /* if blank, go on to scan for body */
	      if (cnt == 0)
		break;

	      /* if non-blank, check for useful headers */
	      if (is_token (input + dummy, cnt,
			    content_length, content_length_len, 0))
		s->content_length =
		  atol ((char *) input + dummy + content_length_len);

	      continue;
	    }

	  /* incomplete line, input is ended for now.  return and try again
	     when there's more data */

	  if (done == 0)
	    return;

	  /* incomplete line, input stream has reached eof mid-headers.
	     this is an error */

	  www_reply (fd, 1000, 400, bad_request, NULL, NULL, 0);
	  mp_delete_read (fd);
	  xbuffer_free (&(s->xb));
	  free (s);
	  return;
	}

      if (s->content_length)
	{
	  /* find first non-blank line */

	  while (1)
	    {
	      cnt = scan_line (input, len, &(s->ptr), &(s->body), &dummy_len);

	      /* if there is non-blankness at the beginning of the line,
	         this is the beginning of the body */

	      if (cnt)
		{
		  /* s->body not set above if the line is incomplete */
		  if (!dummy_len)
		    s->body = s->ptr;

		  break;
		}

	      /* if this is a blank line, go on to the next one */

	      if (dummy_len)
		continue;

	      /* the end of input has been reached for now and there's
	         nothing interesting.  back up one char so that the
	         next call to reader() sees a blank line, and try again
	         when there's more data */

	      if (done == 0)
		{
		  s->ptr--;
		  return;
		}

	      /* the input stream got eof before the body started.
	         this is an error */

	      www_reply (fd, 1000, 400, bad_request, NULL, NULL, 0);
	      mp_delete_read (fd);
	      xbuffer_free (&(s->xb));
	      free (s);
	      return;
	    }
	}

      /* when this point is passed, s->body is set above to the
         first line of the body.  Thus, this block will never be
         entered again */
    }

  if (s->content_length)
    {
      if ((len - s->body) < s->content_length)
	{
	  /* if there's not enough data, try again when there's more */
	  if (done == 0)
	    return;

	  /* if the input stream has ended, and there's not enough
	     body, that's an error */

	  www_reply (fd, 1000, 400, bad_request, NULL, NULL, 0);
	  mp_delete_read (fd);
	  xbuffer_free (&(s->xb));
	  free (s);
	  return;
	}
    }

  {
    char buf[1024];

    sprintf (buf, "request received: %.*s%s %.*s%s",
	     (int) ((s->method_len <= 50) ? s->method_len : 50),
	     input + s->method, (s->method_len <= 50) ? "" : " (truncated)",
	     (int) ((s->uri_len <= 850) ? s->uri_len : 850), input + s->uri,
	     (s->uri_len <= 850) ? "" : " (truncated)");

    log_info ("reader [www]", buf);
  }


  /* handle the request */

  if (is_token (input + s->method, s->method_len, get, get_len, 0))
    {
      if (s->content_length > 0)
	{
	  www_reply (fd, 1000, 400, bad_request, NULL, NULL, 0);
	  mp_delete_read (fd);
	  xbuffer_free (&(s->xb));
	  free (s);
	  return;
	}
      else
	{
	  s->body = 0;

	  ptr = scan_char (input + s->uri, s->uri_len, &(s->body), '?',
			   &dummy, &dummy_len);

	  if (ptr < s->uri_len)
	    {
	      s->content_length = s->uri_len - s->body;
	      s->uri_len = ptr;

	      s->body += s->uri;
	    }
	}
    }
  else if (is_token (input + s->method, s->method_len, post, post_len, 0))
    {
      if (s->content_length == 0)
	{
	  www_reply (fd, 1000, 400, bad_request, NULL, NULL, 0);
	  mp_delete_read (fd);
	  xbuffer_free (&(s->xb));
	  free (s);
	  return;
	}
      else if (readonly)
	{
	  www_reply (fd, 1000, 403, "Forbidden", NULL, NULL, 0);
	  mp_delete_read (fd);
	  xbuffer_free (&(s->xb));
	  free (s);
	  return;
	}
    }
  else
    {
      www_reply (fd, 1000, 400, bad_request, NULL, NULL, 0);
      mp_delete_read (fd);
      xbuffer_free (&(s->xb));
      free (s);
      return;
    }

  if (s->content_length == 0)
    {
      (*(s->h)) (fd, 1000, input + s->uri, s->uri_len, NULL, 0, s->c);
    }
  else
    {
      (*(s->h)) (fd, 1000, input + s->uri, s->uri_len,
		 input + s->body, s->content_length, s->c);
    }

  mp_delete_read (fd);
  xbuffer_free (&(s->xb));
  free (s);
}

static void
listener (int fd, unsigned char *input, long len, int done, void *c)
{
  hh_state *s = (hh_state *) c;
  hh_state *hhs;

#ifdef WITH_IPV6
  struct sockaddr_in6 sin;
  char ipv6addr[INET6_ADDRSTRLEN];
#else
  struct sockaddr_in sin;
  unsigned long addr;
#endif
#if HAVE_SOCKLEN_T
  socklen_t sinlen;
#else
  unsigned int sinlen;
#endif
  int srv;
  char buf[1024];

#if HAVE_LIBWRAP
#if WITH_IPV6
  char ip[100];			/* Now need a size for this strings */
#else
  char *ip;
  struct hostent *jostia;
#endif

  char message[1024];
  char strtmp[256];
#endif

  sinlen = sizeof (sin);

  if ((srv = accept (fd, (struct sockaddr *) &sin, &sinlen)) < 0)
    {
      sprintf (buf, "error accepting new www connection: errno = %d", errno);
      switch (errno)
	{
#if ECONNABORTED
	  /* This happens on Solaris and FreeBSD. */
	case ECONNABORTED:
#endif
#if EINTR
	case EINTR:
#endif
#if EPROTO
	  /* This happens on Solaris, but it's transient. */
	case EPROTO:
#endif
#if defined(ECONNABORTED) || defined (EINTR) || defined(EPROTO)
	  /* Logging an error and returning allows the daemon to recover. */
	  log_error ("listener", buf);
	  return;
	  break;
#endif
	default:
	  log_fatal ("listener", buf);
	  break;
	}			/* switch errno */
    }				/* if accept failed */

#if HAVE_LIBWRAP

#if WITH_IPV6

  inet_ntop (AF_INET6, &sin.sin6_addr, ipv6addr, sizeof (ipv6addr));

  /* Eventually, this should do a real reverse lookup. */
  strncpy (strtmp, "", 255);

#else /* IPv4 code */

  /* Host allow control */
  ip = inet_ntoa (sin.sin_addr);
  /* We need to check that the IP HAS reverse IP resolution */
  if ((jostia = gethostbyaddr ((char *) &(sin.sin_addr),
			       sizeof (sin.sin_addr), AF_INET)) != NULL)


    {
      strncpy (strtmp, jostia->h_name, 255);
    }
  else
    {
      strncpy (strtmp, "", 255);
    }

#endif

  if (!hosts_ctl (TCPDSERVICE, strtmp, ip, STRING_UNKNOWN))
    {
      /* Not allowed to connect */
      snprintf (message, 1023,
		"pksd: host %s/%s not allowed to connect to this server",
		strtmp, ip);
      log_error ("listener", message);
      close (srv);
      return;
    }
  /* host allowed to connect */
  snprintf (message, 1023, "pksd: host %s/%s connected", strtmp, ip);
  log_error ("listener", message);
#endif
  if (fcntl (srv, F_SETFD, 1) < 0)
    {
      sprintf (buf, "failed making http connection close-on-exec: %d", errno);
      log_fatal ("www_init", buf);
    }

#ifdef WITH_IPV6
  inet_ntop (AF_INET6, &sin.sin6_addr, ipv6addr, sizeof (ipv6addr));

  snprintf (buf, sizeof (buf), "new www connection from %s", ipv6addr);
#else
  addr = ntohl (sin.sin_addr.s_addr);

  sprintf (buf, "new www connection from %d.%d.%d.%d",
	   (int) (addr >> 24) & 0xff, (int) (addr >> 16) & 0xff,
	   (int) (addr >> 8) & 0xff, (int) addr & 0xff);
#endif

  log_info ("listener [www]", buf);

  if ((hhs = (hh_state *) malloc (sizeof (hh_state))) == NULL)
    log_fatal ("listener", "failed allocating memory for hh_state");

  *hhs = *s;

  if (!mp_add_read (srv, 1, &reader, (void *) hhs))
    log_fatal ("listener", "failed adding new input stream");
}

int
www_init_addr (void *e, void *c)
{
  int s;
#ifdef WITH_IPV6
  struct sockaddr_in6 sin;
  int result;
#else
  struct sockaddr_in sin;
#endif
  char buf[1024];
  hh_state *hhs;
  int one = 1;
  char *addr = (char *) e;
  struct www_listen_info *ctx = (struct www_listen_info *) c;

  if ((hhs = (hh_state *) malloc (sizeof (hh_state))) == NULL)
    log_fatal ("www_init", "failed allocating memory for hh_state");

#ifdef WITH_IPV6
  if ((s = socket (PF_INET6, SOCK_STREAM, 0)) < 0)
#else
  if ((s = socket (PF_INET, SOCK_STREAM, 0)) < 0)
#endif
    {
      sprintf (buf, "failed creating www socket: %d", errno);
      log_fatal ("www_init", buf);
    }

  if (setsockopt (s, SOL_SOCKET, SO_REUSEADDR,
		  (char *) &one, sizeof (one)) < 0)
    {
      sprintf (buf, "failed setting SO_REUSEADDR on www socket: %d", errno);
      log_fatal ("www_init", buf);
    }

  if (fcntl (s, F_SETFD, 1) < 0)
    {
      sprintf (buf, "failed making www socket close-on-exec: %d", errno);
      log_fatal ("www_init", buf);
    }

#ifdef WITH_IPV6
  sin.sin6_family = AF_INET6;
  sin.sin6_port = htons (ctx->port);
#else
  sin.sin_family = AF_INET;
  sin.sin_port = htons (ctx->port);
#endif
  if (addr != NULL && addr[0] != 0)
    {
#ifdef WITH_IPV6
      /*
       * If it's an IPv4 address map it to an IPv6 address.
       */
      if (inet_pton (AF_INET, addr, &sin.sin6_addr))
	{
	  snprintf (buf, sizeof (buf), "::ffff:%s", addr);
	  result = inet_pton (AF_INET6, buf, &sin.sin6_addr);
	}
      else
	{
	  result = inet_pton (AF_INET6, addr, &sin.sin6_addr);
	}

      if (result < 0)
	{
	  sprintf (buf, "failed to parse address to bind to: %d", errno);
	  log_fatal ("www_init", buf);
	}
#else
      sin.sin_addr.s_addr = inet_addr (addr);
#endif
    }
  else
    {
#ifdef WITH_IPV6
      sin.sin6_addr = in6addr_any;
#else
      sin.sin_addr.s_addr = htonl (INADDR_ANY);
#endif
    }

  if (bind (s, (struct sockaddr *) &sin, sizeof (sin)) < 0)
    {
      sprintf (buf, "failed binding www socket to port %d: %d", ctx->port,
	       errno);
      log_fatal ("www_init", buf);
    }

  if (listen (s, 5) < 0)
    {
      sprintf (buf, "failed listening on www socket: %d", errno);
      log_fatal ("www_init", buf);
    }

  hhs->h = ctx->h;
  hhs->c = ctx->c;
  hhs->readonly = ((pks_www_conf *) ctx->c)->readonly;
  xbuffer_alloc (&(hhs->xb));
  hhs->ptr = 0;
  hhs->uri = -1;
  hhs->body = -1;
  hhs->content_length = 0;

  if (!mp_add_listen (s, 0, &listener, (void *) hhs))
    log_fatal ("www_init", "failed adding new listener for www");

  return 1;
}

void
www_init (llist * addrs, int port, httphandler h, void *c)
{
  struct www_listen_info context;

  context.port = port;
  context.h = h;
  context.c = c;

  if (addrs != NULL && llist_count (addrs) > 0)
    {
      llist_iterate (addrs, www_init_addr, &context);
    }
  else
    {
      www_init_addr (NULL, &context);
    }
}
