/*
 * Portions copyright 2002, 2003 Richard Laager.
 * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Marc Horowitz.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MARC HOROWITZ ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL MARC HOROWITZ BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

const char rcsid_wwwtest_c[] =
  "$Id: wwwtest.c,v 1.6 2003/07/31 18:45:18 rlaager Exp $";

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "globals.h"
#include "multiplex.h"
#include "util.h"
#include "www.h"

/* This is a dummy for log_fatal.  there isn't actually a database
   to close */

int
kd_close ()
{
  return (1);
}

void
wwwtest (int fd, int vers, unsigned char *uri, long urilen,
	 unsigned char *body, long bodylen, void *c)
{
  xbuffer xb;

  xbuffer_alloc (&xb);

  if (!xbuffer_append_str (&xb, "uri: ") ||
      !xbuffer_append (&xb, uri, urilen) ||
      !xbuffer_append_str (&xb, "<br>body:") ||
      !xbuffer_append (&xb, body, bodylen) ||
      !xbuffer_append_str (&xb, "\015\012"))
    {
      xbuffer_free (&xb);
      log_fatal ("wwwtest", "appending to www reply");
    }

  www_reply (fd, vers, 200, "OK", "text/plain", xb.buf, xb.len);

  xbuffer_free (&xb);
}

int
main (int argc, char *argv[])
{
  if (!mp_init ())
    log_fatal ("main", "failed initializing multiplexor");

  www_init (NULL, 8888, wwwtest, NULL);
  www_init (NULL, 9999, wwwtest, NULL);

  printf ("wwwtest server started\n");

  if (!mp_go ())
    log_fatal ("main", "error in multiplexor main loop");


  exit (0);
}
